#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Feb  7 08:23:26 2021

@author: amacquet

Compute the efficient detection volume and rate upper limit for ECBC waveforms (NCSACAM).
"""

import yaml
import numpy as np
from scipy.optimize import curve_fit
from tabulate import tabulate

letters = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I']
waveforms = ['NCSACAM_' + letter for letter in letters]

# Best pipeline for each waveform
bestpip_O3= ['cWB', 'cWB', 'Zebragard_O3a', 'cWB', 'cWB', 'Zebragard_O3a', 'Zebragard_O3a', 'cWB', 'cWB']

def sigmoid(x, a, k , x0):
    return a / (1 + np.exp(k * (x - x0)))

def fit_efficiency(hrss, eff, model='sigmoid'):

    if model == 'sigmoid':
        p0 = [1, 5, np.mean(np.log10(hrss))]
        func = sigmoid
            
    try:
        p0, _ = curve_fit(func, np.log10(hrss), eff, p0=p0)

    except RuntimeError:
        print('Fit did not converge.')
        p0[2] = 1

    return p0


def get_val50(x, y):
    
    y2 = np.abs(y - 0.5)
    amin = np.argmin(y2)
    return x[amin]

def compute_efficient_volume(i):
    """Compute the efficient detection volume V for a given waveform.
    
    V = integral[0, +inf] (4 pi r^2 eff(r) dr)
    
    Output : Volume [Mpc^3]
    """
    # Read efficiency results
    fname = bestpip_O3[i] + '/' + waveforms[i] + '.dat'
    if bestpip_O3[i] == 'cWB':
        fname = bestpip_O3[i] + '/eff_' + waveforms[i] + '.txt'

    data = np.loadtxt(fname, usecols=(0,3))
    hrss = data[:,0]
    eff = data[:,1]

    # Convert hrss into distance
    wvf_params = yaml.load(open('waveforms/' + waveforms[i] +\
                                '.yml', 'r'), Loader=yaml.FullLoader)    
    hrss0 = wvf_params['HRSS']
    dist0 = wvf_params['DISTANCE']

    dist = dist0 * hrss0 / hrss
   
    # Fit efficiency curve
    p0_dist = fit_efficiency(dist, eff)
    p0_hrss = fit_efficiency(hrss, eff)
    
    distances = np.arange(0.1,1000,0.1)
    hrsss = dist0 * hrss0 / distances
    eff_dist = sigmoid(np.log10(distances), p0_dist[0], p0_dist[1], p0_dist[2])
    eff_hrss = sigmoid(np.log10(hrsss), p0_hrss[0], p0_hrss[1], p0_hrss[2])
    
    # Compute efficient detection volume (int 4 pi r^2 eff(r))
    Veff1 = 4 * np.pi * np.trapz(eff_dist*distances**2, dx=0.1)
    Veff2 = -4 * np.pi * (dist0 * hrss0)**3 * np.trapz(eff_hrss*hrsss**(-4), x = hrsss)

    if not np.isclose(Veff1, Veff2, rtol = 1e-3):
        print('Warning : the two integrals do not give the same result.')

    # Compute hrss@50% (to compare with pipeline results)
    hrss50 = get_val50(hrsss, eff_hrss)
    print('hrss@50% for ' + waveforms[i] + ' : ' + format(hrss50, '.2e'))

    return Veff1
    
table = np.zeros(((len(waveforms), 2)), dtype='<U50')

# O3 total observing time (yr)
Tobs = 204.7 / 365.25
for i in range(len(waveforms)):
    table[i,0] = waveforms[i]
    
    v = compute_efficient_volume(i)
    
    # Compute rate upper limit in Gpc^-3 yr^-1
    table[i,1] = 2.3*10**9 / (v * Tobs) 

# print in table
s = tabulate(table, tablefmt='latex', floatfmt='.2e', headers=('Waveform', 'Rate upper limit'))
print(s)
